//
//  ApplicationController.m
//  DemoApp
//
//  Created by Stephen Anderson on 11/10/13.
//  Copyright (c) 2013 Product & Technology. All rights reserved.
//

#import "ApplicationController.h"
#import "xAdSdk.h"

static const NSUInteger NUMBER_OF_ORGANICES_PER_REQUEST = 12;

@implementation ApplicationController

+ (ApplicationController*) sharedInstance
{
    static ApplicationController *singleton = nil;
    
    @synchronized(self)
    {
        NSLog(@"ApplicationController::sharedInstance");
        if (singleton) {
            return singleton;
        }
        
        singleton = [[ApplicationController alloc] init];
        
        [[NSNotificationCenter defaultCenter] addObserver:singleton
               selector: @selector(onSDKError:)
                   name: xAdEventSDKError
                 object: nil];

        [[NSNotificationCenter defaultCenter] addObserver:singleton
               selector: @selector(onSearchAdDelivered:)
                   name: xAdEventSearchCompleted
                 object: nil];

        
        return singleton;
    }
}


- (void) onSearchAdDelivered:(NSNotification*)notification {
    NSLog(@"ApplicationController::onSearchAdDelivered ");
    self.searchResults = notification.object;
    
    NSAssert(self.delegate, @"Opps: The delegate was not set.");
    
    // Application specific selector - not related to the SDK - pick what you want.
    [self.delegate performSelector:@selector(onSearchAdDelivered:) withObject:notification.object];
}


+ (void) startNewSearch:(id)searchKeyword delegate:(id)client
{
    NSLog(@"ApplicationController::startNewSearch(%@, %@)", searchKeyword, client);

    ApplicationController *controller = [ApplicationController sharedInstance];

    xAdLocalSearchRequest *request = [[xAdLocalSearchRequest alloc] init];
    
#warning Make sure the traffic source is agreed upon with xAd. It influences reporting.
    [request setTrafficSource:@"DemoAppSearch"];
    
#warning Manually setting a location overwrites GPS readings.
    [request setCityStateOrZip:@"95014"];
    
    [request setUserGender: GenderMale];
    [request setUserAge: 37];
    
    [request enableOrganicListings];
    [request setNumberOfOrganicResults: NUMBER_OF_ORGANICES_PER_REQUEST];
    [request setRadius:5];
    
    [request setSearchQuery: searchKeyword];
    
    controller.searchRequest = request;
    controller.searchTerm = searchKeyword;
    controller.delegate = client;
    controller.searchResults = nil; 
    
    [request execute];
}


+ (void) startLocalSearch:(NSUInteger)startIndex delegate:(id)client
{
    NSLog(@"ApplicationController::startLocalSearch(%d, %@)", startIndex, client);

    ApplicationController *controller = [ApplicationController sharedInstance];
    
    if ([controller.searchTerm length] == 0) {
        // Nothing to search, only applicable to local search.
        return;
    }
    
    xAdLocalSearchRequest *request = controller.searchRequest;
    
    // All other paraters stay the same
    
    [request setStartIndex:startIndex];
    
    // Set new delegate because NEXT/PREV will come from a different page
    controller.delegate = client;
    controller.searchResults = nil;

    [request execute];
}


+ (void) nextPageSearchWithDelegate:(id)client
{
    NSLog(@"ApplicationController::nextPageSearchWithDelegate ");

    ApplicationController *controller = [ApplicationController sharedInstance];
    
    NSInteger newStartIndex = controller.searchResults.organicsStartIndex + NUMBER_OF_ORGANICES_PER_REQUEST;
    
    [ApplicationController startLocalSearch:newStartIndex delegate:client];
}


+ (void) prevPageSearchWithDelegate:(id)client
{
    NSLog(@"ApplicationController::prevPageSearchWithDelegate ");

    ApplicationController *controller = [ApplicationController sharedInstance];
    
    NSInteger newStartIndex = controller.searchResults.organicsStartIndex - NUMBER_OF_ORGANICES_PER_REQUEST;
    
    if (newStartIndex < 0) {
        newStartIndex = 0;
    }
    
    [ApplicationController startLocalSearch:newStartIndex delegate:client];
}








@end
