//
//  SearchViewController.m
//  DemoApp
//
//  Created by Stephen Anderson on 11/9/13.
//  Copyright (c) 2013 Product & Technology. All rights reserved.
//

#import "SearchViewController.h"
#import "xAdNotificationRequest.h"
#import "xAdListing.h"
#import "ApplicationController.h"




@implementation SearchViewController
@synthesize searchResults;
@synthesize request;
@synthesize searchTerm;


- (void) setSearchResults:(xAdLocalSearchResults *)results {

#warning Important to send impression notifications for each search result. Overwriting the setter we can resend the notification everytime we have new results. Keep in mind paging.
    
    searchResults = results;
    
    self.buttonNext.enabled = results.isNextAvailable;
    self.buttonPrevious.enabled = results.isPrevAvailable;
    
    [searchResults sendImpressionNotifications];
    
    [self.tableView reloadData];
}





- (void)viewDidLoad
{
    [super viewDidLoad];

    ApplicationController *controller = [ApplicationController sharedInstance];
    
    self.title = [NSString stringWithFormat:@"Results for %@", controller.searchTerm];
    
    self.searchResults = controller.searchResults;
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source



- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section{
    switch (section) {
        case 0:
            return @"Sponsored";
            
        case 1:
            return @"Organics";
            
        default:
            return 0;
    }}


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    if (self.searchResults == nil) {
        return 0;
    }
    
    NSUInteger sections = 0;
    
    if (self.searchResults.hasSponsoredListings) {
        sections++;
    }
    
    if (self.searchResults.hasOrganicListings) {
        sections++;
    }
    
    return sections;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    switch (section) {
        case 0:
            return self.searchResults.sponsoredListings.count;
            
        case 1:
            return self.searchResults.organicListings.count;
            
        default:
            return 0;
    }
}


- (xAdListing*) objectAtIndexPath:(NSIndexPath *)indexPath {
    
    switch (indexPath.section) {
        case 0:
            return [self.searchResults.sponsoredListings objectAtIndex:indexPath.row];
            
        case 1:
            return [self.searchResults.organicListings objectAtIndex:indexPath.row];
            
        default:
            return nil;
    }
}



- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];

    if(cell == nil)
    {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
    }
    
    xAdListing *object = [self objectAtIndexPath:indexPath];
    
    cell.textLabel.text = object.name;
    cell.detailTextLabel.text = object.formattedPhone;
    
    return cell;
}



- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {

    xAdListing *object = [self objectAtIndexPath:indexPath];
    
    [object sendClickNotification];
    
#warning Observe the Action Type
    if (object.actionType == ActionTypeClick) {
        // Open details page
    } else if (object.actionType == ActionTypeUrl) {
        // Open a webview
    }
    
}



/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

/*
#pragma mark - Navigation

// In a story board-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}

 */

#pragma mark Simple Paging


- (IBAction)onNextTapped:(id)sender {
    [ApplicationController nextPageSearchWithDelegate:self];
}


- (IBAction)onPreviousTapped:(id)sender {
    [ApplicationController prevPageSearchWithDelegate:self];
}


- (void) onSearchAdDelivered:(xAdLocalSearchResults*) results {
    self.searchResults = results;
}


@end
