//
//  xAdBannerView.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//


#import "xAdBannerView.h"
#import "xAdDisplayAPI.h"
#import "xAdLandingPageViewController.h"
#import "xAdSdk.h"


@interface xAdBannerView () <UIWebViewDelegate>

@end


@interface xAdBannerView ()

@property (nonatomic, strong) id delegate;
@property (nonatomic, strong) id bannerSize;

@end


@implementation xAdBannerView
@synthesize delegate;
@synthesize webview;





- (UIViewController*) parentController {
    return self.delegate;
}

- (id)initWithDelegate:(id)d size:(BannerSize)size placement:(CGPoint)point {

    CGRect frame = CGRectMake(point.x, point.y, 0, 0);
    
    switch (size) {
        case BannerSize300x250:
            frame.size = CGSizeMake(320, 250);
            self.bannerSize = @"300x250";
            break;
        case BannerSize300x50:
            frame.size = CGSizeMake(320, 50);
            self.bannerSize = @"300x50";
            break;
        case BannerSize320x50:
            frame.size = CGSizeMake(320, 50);
            self.bannerSize = @"320x50";
            break;
        case BannerSize728x90:
            frame.size = CGSizeMake(728, 90);
            self.bannerSize = @"728x90";
            break;
            
        default:
            break;
    }

    return [self initWithDelegate:d viewFrame:frame];
}




- (id)initWithDelegate:(id)d
             viewFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    
    if (self) {
        self.delegate = d;
        self.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
        
        // Since we support iOS 6+
        self.webview = [[UIWebView alloc] initWithFrame:CGRectMake(0, 0, frame.size.width, frame.size.height)];
        self.webview.delegate = self;
        self.webview.backgroundColor = [UIColor clearColor];
        self.webview.opaque = NO;
        self.webview.clipsToBounds = YES;
        self.webview.scrollView.bounces = NO;
        self.webview.scrollView.scrollEnabled = NO;
        [self.webview setAllowsInlineMediaPlayback:YES];
        [self.webview setMediaPlaybackRequiresUserAction:NO];

        [self addSubview:self.webview];
    }
    
    return self;
}


#pragma mark REQUEST NEW AD

-(void) requestAd:(NSDictionary*)targetingParameters
{
    [xAdDisplayAPI requestAd:self.bannerSize args:targetingParameters delegate:self];
}


#pragma mark xAdDisplayAPIDelegate

- (void) onDisplayAdDelivered:(NSString*)htmlPayload {
    NSAssert(NO, @"xAdBannerView not derived");
}


- (void) onDisplayAdError:(NSError*)error {
    NSAssert(NO, @"xAdBannerView not derived");
}


- (void) onDisplayAdNoFill {
    NSAssert(NO, @"xAdBannerView not derived");
}




#pragma mark INTERCEPTS BANNER CLICKS TO LAUNCH THE LANDING PAGE

- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType
{
    if(navigationType != UIWebViewNavigationTypeLinkClicked)
        return YES;
    
    NSString *urlString = request.URL.absoluteString;
    NSLog(@"Banner open: %@", urlString);

    if ([request.URL.scheme isEqualToString:@"applewebdata"]) {
        NSLog(@"Invalid CLICK url 'applewebdata'.");
        return NO;
    }
    else if ([request.URL.absoluteString isEqualToString:@""]) {
        NSLog(@"Invalid CLICK url, its blank.");
        return NO;
    }
    else if ([urlString rangeOfString:@"itunes.apple.com" options:NSCaseInsensitiveSearch range:NSMakeRange(0,0)].location != NSNotFound)
    {
        if ([[UIApplication sharedApplication]canOpenURL:request.URL]) {
            [[UIApplication sharedApplication]openURL:request.URL];
            return NO;
        }
    }
    
    // Send click notification to all parties interected - publisher should listen
    // for clicks for their own analytic efforts.
    [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventDisplayAdClicked object:nil];
    
    // Launch landing page on the controller of the publisher. Relies on the xAdDelegate.
    UINavigationController *controller = (UINavigationController*)[self parentController];
    xAdLandingPageViewController *lpController = [xAdLandingPageViewController createWithRequest:request];
    [controller presentViewController:lpController animated:YES completion:nil];
    
    return NO; // Prevent webview from rendering
}



@end
