//
//  xAdDisplayAPI.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import "xAdDisplayAPI.h"
#import "xAdUtils.h"

#import "xAdSdk.h"
#import "xAdBannerView.h"


#define PRODUCTION_DISPLAY_ELB @"http://display.xad.com/rest/banner?%@"
#define TEST_CHANNEL_URL @"http://creative.xadcentral.com:8080/testchannel/%@"





@implementation xAdDisplayAPI


+ (void) requestAd:(NSString*)bannerSize args:(NSDictionary*)args delegate:(id<xAdDisplayAPIDelegate>)delegate
{
    NSLog(@"requestBanner");
    
    NSMutableDictionary *params = [NSMutableDictionary dictionaryWithDictionary:args];

    [params setObject:bannerSize forKey:xAdParameterDisplaySize];
    
    BOOL hasLocationInformation = FALSE;
    
    id userLocation = [args objectForKey:xAdParameterGeoCityState];
    
    if (userLocation) {
        hasLocationInformation = TRUE;
    }
    
    id userInternetAddress = @"0.0.0.0";
    
    if (userInternetAddress && ![userInternetAddress isEqualToString:@"0.0.0.0"]) {
        [params setObject:userInternetAddress forKey:xAdParameterDeviceIp];
        hasLocationInformation = TRUE;
    }

    CLLocation* currentLocation = [xAdSdk currentLocation];

    if (currentLocation && currentLocation.coordinate.latitude != 0.0 && currentLocation.coordinate.longitude != 0.0)
    {
        hasLocationInformation = TRUE;
        NSString *lat = [NSString stringWithFormat:@"%f", currentLocation.coordinate.latitude];
        NSString *lon = [NSString stringWithFormat:@"%f", currentLocation.coordinate.longitude];
        
        [params setObject:lat forKey:xAdParameterGeoLat];
        [params setObject:lon forKey:xAdParameterGeoLong];
    }

    if (!hasLocationInformation)
    {
        NSLog(@"WARNING: No location available OR location services turned off.");
        
        id details =[NSDictionary dictionaryWithObjectsAndKeys:@"No location available OR location services turned off.", NSLocalizedDescriptionKey, nil];
        
        NSError *error = [NSError errorWithDomain:@"com.xad.sdk" code:10 userInfo:details];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventSDKError object:error];

        return;
    }

    xAdSdk *sdk = [xAdSdk sharedInstance];
    
    NSDictionary *autoParams = @{
                             xAdParameterPublisherKey: sdk.displayPublisherKey,
                             xAdParameterVersion: @"1.1",

                             xAdParameterDeviceCarrier: [xAdUtils carrier],
                             xAdParameterDeviceUserAgent: [xAdUtils userAgent],
                             xAdParameterDeviceType: [xAdUtils deviceType],
                             xAdParameterDeviceModel: [xAdUtils deviceModel],
                             xAdParameterDeviceOS: [xAdUtils deviceOS],
                             xAdParameterDeviceLanguage: [xAdUtils deviceLanguage],
                             xAdParameterDeviceWifiProvider: [xAdUtils deviceWifi],
                             
                             xAdParameterUserId: [xAdUtils advertisingIdentifier],
                             xAdParameterUserIdType: @"IDFA|RAW",
                             xAdParameterUserDoNotTrack: [xAdUtils isDoNotTrackEnabled],
                             xAdParameterPublisherType:@"app"
                             };
    
    [params addEntriesFromDictionary:autoParams];
    
    NSString *urlArgs = [xAdUtils toURL:params urlEncode:YES];
    
    NSString *aResource = [NSString stringWithFormat:sdk.displayServer, urlArgs];
    NSLog(@"%@", aResource);
    
    NSURL *url = [NSURL URLWithString:aResource];
    
    dispatch_async(KBackgroundQueue, ^(void) {
        NSError *networkError = nil;
        NSData* data = [NSData dataWithContentsOfURL:url options:NSDataReadingUncached error:&networkError];
        
        NSUInteger payloadSize = 0;
        NSString* htmlPayload = nil;
        
        if (!networkError) {
            payloadSize = [data length];
            if (payloadSize) {
                htmlPayload = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] ;
            }
        }
        
        dispatch_async(KForegroundQueue, ^(void) {
            
            if (networkError) {
                [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventSDKError object:networkError];
                [delegate onDisplayAdError:networkError];
            }
            else
            {
                if (payloadSize == 0) {
                    [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventDisplayAdNoFill object:nil];
                    [delegate onDisplayAdNoFill];
                }
                else {
                    [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventDisplayAdFilled object:nil];
                    [delegate onDisplayAdDelivered:htmlPayload];
                }
            }
            
        });
        
    });

}

@end









