//
//  xAdLandingPageViewController.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import "xAdLandingPageViewController.h"
#import "xAdDisplayAPI.h"
#import "xAdSdk.h"
#import <objc/runtime.h>
#import <PassKit/PassKit.h>
#import "xAdUtils.h"

static char KEY_ALERTVIEW_COOKIE;

@interface xAdLandingPageViewController ()
@property (strong, nonatomic) IBOutlet UIWebView *webView;
@property (strong, nonatomic) NSURLRequest *request;

- (IBAction)onClose:(id)sender;
- (IBAction)onBack:(id)sender;

@end

@implementation xAdLandingPageViewController
@synthesize webView;
@synthesize request;


+ (id) createWithRequest:(NSURLRequest*)request {
    return [[xAdLandingPageViewController alloc] initWithRequest:request];
}




- (id)initWithRequest:(NSURLRequest*)req
{
    self = [super initWithNibName:@"xAdLandingPageViewController" bundle:nil];
    
    if (self) {
        
        id currentUrl = req.URL.absoluteString;
        
        if ([currentUrl rangeOfString:@"ap.xad.com"].location != NSNotFound) {
            currentUrl = [currentUrl stringByReplacingOccurrencesOfString:@"&sdk_conf=&" withString:@"&sdk_conf=jssdk&"];
        }

        NSURLRequest *r = [[NSURLRequest alloc] initWithURL:[NSURL URLWithString:currentUrl]];
        
        self.request = r;
    }
    return self;
}

- (IBAction)onClose:(id)sender {
    if (![self.presentedViewController isBeingDismissed])
        [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)onBack:(id)sender {
    [self.webView goBack];
}


- (void)viewDidLoad
{
    [super viewDidLoad];

    [self.webView setDelegate:self];
    [self.webView setScalesPageToFit:YES];
    [self.webView loadRequest:self.request];

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= MP_IOS_4_0
    if ([self.webView respondsToSelector:@selector(allowsInlineMediaPlayback)]) {
        [self.webView setAllowsInlineMediaPlayback:YES];
        [self.webView setMediaPlaybackRequiresUserAction:NO];
    }
#endif
}


- (BOOL) processRequestWithConfirmation:(NSURLRequest*)req
{
    NSMutableString *userMessage = nil;
    
    if ([req.URL.scheme isEqualToString:@"tel"]) {
        

         
         NSString *r = req.URL.absoluteString;
         
         id n = [r stringByReplacingOccurrencesOfString:@"tel" withString:@"telPrompt"];
         
         req = [NSURLRequest requestWithURL:[NSURL URLWithString:n]];
        
            NSLog(@"URL:%@", req.URL.absoluteString);
        
         [[UIApplication sharedApplication] openURL:req.URL];
         return NO;
         
        
    }
    else if ([req.URL.scheme isEqualToString:@"pass"]) {
        userMessage = [NSMutableString stringWithString:@"Download pass?"];
 
    }
    else {
        return YES; // Unsupported scheme...check schemesRequiringConfirmation
    }
    
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@""
                                                                 message:userMessage
                                                                delegate:self
                                                       cancelButtonTitle:@"Cancel"
                                                       otherButtonTitles:@"Call", nil];
    
    objc_setAssociatedObject(alertView, &KEY_ALERTVIEW_COOKIE, req, OBJC_ASSOCIATION_RETAIN);

    [alertView show];
    
    return NO; // Unless specific commands require rendering too
}


- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex == alertView.cancelButtonIndex)
        return;
    
    NSURLRequest *req = (NSURLRequest*)objc_getAssociatedObject(alertView, &KEY_ALERTVIEW_COOKIE);

    [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventDisplayAdSecondary
                                                        object:@"call"];

    [[UIApplication sharedApplication] openURL:req.URL];
}







- (NSArray*) schemesRequiringConfirmation {
    return [NSArray arrayWithObjects:@"tel", @"pass", nil];
}

//
// REQUEST INTERCEPTOR
//
- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)req navigationType:(UIWebViewNavigationType)navigationType
{
//    if(navigationType != UIWebViewNavigationTypeLinkClicked)
//        return YES;
    
    NSLog(@"LP open: %@", req.URL.absoluteString);
    
    if ([req.URL.absoluteString hasPrefix:@"itms-appss://itunes.apple.com"] ||
        [req.URL.absoluteString hasPrefix:@"itmss://itunes.apple.com"])
    {
        [[UIApplication sharedApplication]openURL:req.URL];
        [self dismissMe];
        return NO;
    }
    else if ([req.URL.scheme isEqualToString:@"tel"])
    {
        NSString *r = req.URL.absoluteString;
        
        id substitutedUrl = [r stringByReplacingOccurrencesOfString:@"tel" withString:@"telPrompt"];
        
        req = [NSURLRequest requestWithURL:[NSURL URLWithString:substitutedUrl]];
        
        NSLog(@"URL:%@", req.URL.absoluteString);
        
        [[UIApplication sharedApplication] openURL:req.URL];
        return NO;
    }
    else if ([req.URL.absoluteString hasSuffix:@".pkpass"])
    {
        [[UIApplication sharedApplication] openURL:req.URL];
        return NO;
    }

    return YES;
}


- (void)webViewDidStartLoad:(UIWebView *)webView
{

}

- (void) dismissMe {
    if (![self.presentedViewController isBeingDismissed])
        [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{
    NSLog(@"LP Error: %@", error.localizedDescription);
    
    if (error.code == 102) {
        // frame load interrupted
        return;
    }
    
 //   [self dismissMe];
}


- (void)webViewDidFinishLoad:(UIWebView *)sender
{
    self.webView.hidden = NO;
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}



- (void)viewDidUnload {
    [self setWebView:nil];
    [super viewDidUnload];
}
@end
