//
//  xAdNotificationRequest.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//


#import "xAdNotificationRequest.h"
#import "xAdResponse.h"
#import "xAdListing.h"
#import "xAdSdk.h"
#import "xAdUtils.h"


@implementation xAdNotificationRequest


#pragma mark xAdRESTRequest

- (xAdRequestType) requestType
{
    return RequestTypeNotification;
}

- (NSString*) RESTCommand
{
    return @"notify";
}

- (NSString*) notificationEventName {
    return xAdEventSearchNotifyCompleted;
}

- (xAdResponse*) createResponseFromJson:(NSData*) jsonData
{
    // Notification service responds with a 1x1 image/gif
    return nil;
}

#pragma mark -



- (id) initType:(NotificationType)type listingId:(id)listingId
{
    self = [super init];
    
    if (self)
    {
        xAdSdk *sdk = [xAdSdk sharedInstance];
        
        NSDictionary *defaultValues = @{
                                        xAdParameterSearchRequestType: [self notificationValue: type],
                                        xAdParameterVersion: @"1.1",
                                        xAdParameterPublisherKey: sdk.searchPublisherKey,
                                        xAdParameterSearchListingId: [NSArray arrayWithObject:listingId],
                                        xAdParameterAdFormat: @"json",
                                        xAdParameterUserId: [xAdUtils advertisingIdentifier]
                                        };
        
        [self.fields addEntriesFromDictionary: defaultValues];
    }
    
    return self;
}


- (id) initImpressionWithListingIds:(NSArray*)listOfIds
{
    if (listOfIds.count == 0)
        return nil;
    
    self = [super init];
    
    if (self)
    {
        xAdSdk *sdk = [xAdSdk sharedInstance];
        
        NSDictionary *defaultValues = @{
                                        xAdParameterSearchRequestType:@"imp",
                                        xAdParameterVersion: @"1.1",
                                        xAdParameterPublisherKey: sdk.searchPublisherKey,
                                        xAdParameterSearchListingId: listOfIds,
                                        xAdParameterAdFormat: @"json",
                                        xAdParameterUserId: [xAdUtils advertisingIdentifier]
                                        };
        
        [self.fields addEntriesFromDictionary: defaultValues];
    }
    
    return self;
}




static NSArray *sNotificationTypes = nil; 


- (id) notificationValue:(NotificationType)type
{
    if (sNotificationTypes == nil)
    {
        // WARNING: Faster, less code, BUT must make sure to keep aligned with the ENUM
        NSString *docList = @"imp,click,call,review,desc,sms,map,direction,arrival,checkin,save_pb,save_ms,callop,moreinfo,email,cal,ml_map,social_share,buy_deal";
        sNotificationTypes = [docList componentsSeparatedByString:@","];
    }

    return [sNotificationTypes objectAtIndex:type];
}







@end
