//
//  xAdRequest.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//


#import "xAdRequest.h"
#import "xAdResponse.h"
#import "xAdSdk.h"
#import "xAdUtils.h"



@implementation xAdRequest
@synthesize fields;


- (id) init {

    self = [super init];
    
    if (self) {
        self.fields = [NSMutableDictionary dictionary];
    }
    
    return self;
}


- (NSString*) RESTCommand
{
    NSAssert(NO, @"Overwrite this method");
    return nil;
}


- (xAdResponse*) createResponseFromJson:(NSData*) jsonData
{
    NSAssert(NO, @"Overwrite this method");
    return nil;
}


- (xAdRequestType) requestType
{
    NSAssert(NO, @"Overwrite this method");
    return RequestTypeUnknown;
}


- (NSString*) notificationEventName {
    NSAssert(NO, @"Overwrite this method");
    return nil;
}


- (void) execute {
    
    NSLog(@"xAdRequest.execute");
    
    xAdSdk *sdk = [xAdSdk sharedInstance];

    NSLog(@"%@", self.fields);
    
    id urlArgs = [xAdUtils toURL:self.fields urlEncode:YES];
    id command = [self RESTCommand];
    
    NSString *aResource = [NSString stringWithFormat:sdk.searchServer, command, urlArgs];
    
    NSLog(@"%@", aResource);
    
    NSURL *url = [NSURL URLWithString:aResource];

    dispatch_async(KBackgroundQueue, ^(void) {
        
        NSError *networkError = nil;
        NSData* data = [NSData dataWithContentsOfURL:url options:NSDataReadingUncached error:&networkError];
        
        NSUInteger payloadSize = 0;
        xAdResponse *response = nil;
        
        if (!networkError) {
            payloadSize = [data length];
            if (payloadSize) {
                response = [self createResponseFromJson:data];
            }
        }
        
        BOOL serviceError = FALSE;
        
        if (response) {
            // Not all responses are JSON object. In that case the response is nil, and that is OK.
            serviceError = !response.isSuccess;
        }
        
        dispatch_async(KForegroundQueue, ^(void) {
            
            if (networkError) {
                [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventSDKError
                                                                    object:networkError];
            }
            else if (serviceError) {
                [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventSDKError
                                                                    object:[response createError]];
            }
            else
            {
                [[NSNotificationCenter defaultCenter] postNotificationName:[self notificationEventName]
                                                                    object:response];
            }
            
        });
        
    });
    
 
}













@end


