//
//  xAdListing.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import "xAdListing.h"
#import "NSDictionaryExtension.h"
#import "NSStringExtension.h"
#import "xAdUtils.h"


@implementation xAdListing
@synthesize reviews = _reviews;
@synthesize type = _type;
@synthesize paymentTypes = _paymentTypes;
@synthesize location = _location;
@synthesize urlObjects = _urlObjects;

- (id) initWithFields:(NSDictionary*)fields
{
    NSAssert(NO, @"WARNING: Use 'initWithFields: withType:' instead.");
    return nil;
}


- (id) initWithFields:(NSDictionary*)fields withType:(ListingType)type
{
    self = [super initWithDictionary:fields];
    
    if (self)
    {
        _type = type;
    }
    
    return self;
}



- (NSString*) listingID
{
    return [self.fields stringForKey:@"l_id"];
}


- (NSString*) name
{
    return [self.fields stringForKey:@"name"];
}


- (NSDictionary*) moreInfo
{
    return [self.fields dictionaryForKey:@"more_info"];
}


- (NSString*) shortDescription
{
    return [self.fields stringForKey:@"s_desc"];
}


- (id) fullDescription
{
    id eDesc = [self.moreInfo stringForKey:@"e_desc"];
    id lDesc = [self.fields stringForKey:@"l_desc"];
    id sDesc = [self.fields stringForKey:@"s_desc"];
    
    id value = nil;
    
    if (eDesc && [eDesc length] > 0)
        value = eDesc;
    else if (lDesc && [lDesc length] > 0)
        value = lDesc;
    else if (sDesc && [sDesc length] > 0)
        value = sDesc;
    else
        value = @"No information available.";
    
    return [value stringByStrippingHTML];
}


- (NSArray*) urlObjects
{
    if (_urlObjects)
        return _urlObjects;
    
    id complexObject = [self urls];
    
    if(complexObject== nil)
        return nil;
    
    _urlObjects = [xAdUtils objectsFromObject:complexObject];
    
    return _urlObjects;
}


- (id) urlOfType:(NSString*)filter
{
    NSArray *urlObjects = [self urlObjects];
    
    for (NSDictionary *object in urlObjects)
    {
        if ([[object stringForKey:@"type"] isEqualToString:filter])
        {
            id linkAddress = [object stringForKey:@"click"];
            return linkAddress;
        }
    }
    
    return nil;
}


- (NSArray*) urlsOfType:(id)filter
{
    NSArray *links = [self urlObjects];
    
    if (links == nil)
        return nil;
    
    NSMutableArray *results = [NSMutableArray array];
    
    for (NSDictionary *link in links)
    {
        if ([[link objectForKey:@"type"] isEqual:filter])
        {
            [results addObject:[link objectForKey:@"click"]];
        }
    }
    
    return results;
}


- (NSURL*) imageUrl
{
    NSArray *imagesLinks = [self urlsOfType:[self fieldKeyForObjectImage]];
    
    if ([imagesLinks count] == 0)
        return nil;
    
    NSString* urlEncoded = [[imagesLinks objectAtIndex:0] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    
    return [NSURL URLWithString:urlEncoded];
}


- (BOOL) hasPhoneNumber
{
    id redirectedNumber = [self.fields stringForKey:@"phone"];
    id businessNumber = [self.fields stringForKey:@"src_phone"];
    
    return (redirectedNumber || businessNumber);
}


- (NSString*) sourcePhone
{
    return [self.fields stringForKey:@"src_phone"];
}


- (NSString*) phone
{
    id phone = [self.fields stringForKey:@"phone"];
    id src_phone = [self.fields stringForKey:@"src_phone"];
    
    if (phone) {
        return phone;
    } else if (src_phone) {
        return src_phone;
    } else {
        return nil;
    }
}


- (id) formattedPhone {
    
    // Unfortunately we have seen discrepancies with different versions of the API.
    
    id fmt_phone = [self.fields stringForKey:@"fmt_phone"];
    id d_phone = [self.fields stringForKey:@"fmt_phone"];
    id phone = [self.fields stringForKey:@"phone"];

    if (fmt_phone) {
        return fmt_phone;
    } else if (d_phone) {
        return d_phone;
    } else if (phone) {
        return phone;
    } else {
        return nil;
    }
}


- (BOOL) hasCoordinates
{
    return (self.location.geoLocation != nil);
}


- (xAdLocation*) location
{
    if (_location) {
        return _location;
    }
    
    if (self.fields == nil || [self.fields count] == 0) {
        return nil;
    }
    
    id country = @""; // Defaults to USA
    id distance = [self.fields objectForKey:@"dist"];
    
    if (distance == nil) {
        distance = @0.0;
    }
    
    id streetNo = [self.fields objectForKey:@"s_num"];
    id streetNoValue = [NSString stringWithFormat:@"%@", [self.fields objectForKey:@"s_num"]];
    id streetName = [self.fields objectForKey:@"s_name"];
    
    BOOL hasStreetNumber = (streetNo != nil && [streetNoValue length] > 0);
    BOOL hasStreetName = (streetName != nil && [streetName length] > 0);
    
    id street = nil;
    if (hasStreetName == NO)
        street = @"";
    else if (!hasStreetNumber && hasStreetName )
        street = streetName;
    else
        street = [NSString stringWithFormat:@"%@ %@", streetNo, streetName];
    
    id latitude = [self.fields objectForKey:@"lat"];
    id longitude = [self.fields objectForKey:@"long"];
    
    NSDictionary *locFields = @{@"city": [self.fields stringForKey:@"city"],
                                @"state": [self.fields stringForKey:@"state"],
                                @"country":country,
                                @"street":street,
                                @"zip": [self.fields stringForKey:@"zip"],
                                @"latitude":latitude,
                                @"longitude":longitude,
                                @"distance":distance };
    
    _location = [[xAdLocation alloc] initWithDictionary:locFields];
    
    return _location;
}




- (NSURL*) attributionClickLink
{
    id a = [self.fields dictionaryForKey:@"attribution"];
    
    id v = [a urlForKey:@"click_url"];
    
    return v;
}


- (NSURL*) attributionImageLink
{
    id a = [self.fields dictionaryForKey:@"attribution"];
    
    id v = [a urlForKey:@"image_url"];
    
    return v;
}






- (id) objectKey
{
    return [NSString stringWithFormat:@"%@%@", self.dataSource, self.dataSourceId];
}


- (id) cityKey
{
    if (self.location.city)
        return self.location.city;
    else
        return @"Unknown city";
}


- (id) fieldKeyForObjectImage
{
    return @"image";
}


- (id) urls
{
    return [self.moreInfo objectForKey:@"url"];
}


- (NSString*) dataSourceId
{
    return [self.fields stringForKey:@"ds_id"];
}


- (NSString*) dataSource
{
    return [self.fields objectForKey:@"ds"];
}


- (NSUInteger) reviewCount
{
    return [self.fields unsignedIntegerForKey:@"r_count"];
}


- (NSArray*) categories // array of strings
{
    NSDictionary *cats = [self.fields dictionaryForKey:@"cats"];
    
    id object = [cats objectForKey:@"cat"];
    
    NSArray *objects = [xAdUtils objectsFromObject:object];
    
    return objects;
}


- (BOOL) hasRating
{
    id object = [self.fields objectForKey:@"rating"];
    
    if (object == nil)
        return NO;
    
    if ([object isKindOfClass:[NSNumber class]] == YES)
        return YES;
    
    if ([object isKindOfClass:[NSString class]] == YES)
         return ([object length] == YES);
    
    return NO;
}


- (NSInteger) rating
{
    NSInteger value = [self.fields integerForKey:@"rating"];
    
    if (value > 5)
        return 5;
    else
        return value;
}


- (UIImage*) stars
{
    if (self.hasRating == NO)
        return nil;
    
    NSString *filename = [NSString stringWithFormat:@"stars_%d.0.png", self.rating];
    
    return [UIImage imageNamed:filename];
}


- (NSString*) sharableInfo
{
    NSMutableString *data = [NSMutableString string];
    
    [data appendString:self.name];
    [data appendString:@"\n\nAddress:\n"];
    [data appendString:self.location.addressInTwoLines ];

    if (self.shortDescription != nil && [self.shortDescription length] > 0)
        [data appendFormat:@"\n\nDescription:\n%@", self.shortDescription];
    
    return data;
}


- (NSString*) sharableInfoHtml
{
    NSMutableString *data = [NSMutableString stringWithString:@"<details>"];
    
    [data appendFormat:@"<summary>%@</summary>", self.name];
    [data appendFormat:@"<address>%@</address>", self.location.addressInTwoLines];

    if (self.shortDescription != nil && [self.shortDescription length] > 0)
        [data appendFormat:@"<p>%@</p>", self.shortDescription];
    
    [data appendString:@"</details>"];
    
    return data;
}


#pragma mark - More Info Fields


- (BOOL) hasHours
{
    NSDictionary *hours = [self.moreInfo dictionaryForKey:@"hrs"];
    id display = [hours stringForKey:@"display"];
    
    return (display != nil && [display isKindOfClass:[NSString class]] && [display length] > 0);
}


- (NSString*) hours
{
    NSDictionary *hours = [self.moreInfo dictionaryForKey:@"hrs"];
    return [hours stringForKey:@"display"];
}


- (NSArray*) paymentTypes
{
    if (_paymentTypes)
        return _paymentTypes;

    NSDictionary *payment = [self.moreInfo dictionaryForKey:@"payment"];
    
    if (payment == nil)
        return nil;
    
    NSArray *types = [payment arrayForKey:@"type"];
    
    if (types == nil)
        return nil;
    
    _paymentTypes = types;
    
    return _paymentTypes;
}


- (ActionType) actionType
{
    if (self.type == ListingTypeOrganic)
        return ActionTypeNone;
    
    id value = [self.fields stringForKey:@"action"];
    
    if ([value isEqualToString:@"click"])
        return ActionTypeClick;
    else if ([value isEqualToString:@"url"])
        return ActionTypeUrl;
    else
        return ActionTypeNone;
}


- (id) actionUrl
{
    if (self.type == ListingTypeOrganic)
        return nil;
    
    if (self.actionType != ActionTypeUrl)
        return nil;
    
    NSDictionary *url = [self.fields dictionaryForKey:@"url"];
    
    return [url stringForKey:@"click"];
}


- (id) attribution
{
    id complexObject = [self.moreInfo objectForKey:@"attribution"];
    
    if (complexObject == nil)
        return @"";
    
    NSArray *attributions = [xAdUtils objectsFromObject:complexObject];
    
    if (attributions == nil || attributions.count == 0)
        return @"";
    
    NSDictionary *attribution = [attributions objectAtIndex:0];
    
    return [attribution stringForKey:@"text"];
}


- (void) sendClickNotification {
    [self sendNotification:NotificationClick];
}


- (void) sendNotification:(NotificationType)type {
    xAdNotificationRequest *request = [[xAdNotificationRequest alloc] initType:type listingId:self.listingID];
    
    [request execute];
}



@end


