//
//  xAdLocalSearchResults.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import "xAdLocalSearchResults.h"
#import "NSDictionaryExtension.h"
#import "xAdNotificationRequest.h"
#import "xAdSdk.h"
#import "xAdListing.h"
#import "xAdUtils.h"


static NSArray *s_responseTypes = nil;


@implementation xAdLocalSearchResults
@synthesize organicListings = _organicListings;
@synthesize sponsoredListings = _sponsoredListings;




- (NSArray*) organicListings
{
    if (_organicListings != nil) {
        return _organicListings;
    }
    
    NSDictionary *listings = [self.results dictionaryForKey:@"organic_listings"];
    
    id complexObject = [listings objectForKey:@"listing"];
    
    NSArray *objects = [xAdUtils objectsFromObject:complexObject];
    
    NSMutableArray *items = [[NSMutableArray alloc] init];
    for (id objectFields in objects)
    {
        id object = [[xAdListing alloc] initWithFields:objectFields withType:ListingTypeOrganic];
        [items addObject:object];
    }
    
    _organicListings = items;
    
    return _organicListings;
}


- (NSArray*) sponsoredListings
{
    if (_sponsoredListings != nil)
        return _sponsoredListings;
    
    NSDictionary *root = self.results;
    
    NSDictionary *listings = [root dictionaryForKey:@"paid_listings"];
    
    id complexObject = [listings objectForKey:@"listing"];
    
    NSArray *objects = [xAdUtils objectsFromObject:complexObject];
    
    NSMutableArray *items = [[NSMutableArray alloc] init];
    for (id objectFields in objects)
    {
        id object = [[xAdListing alloc] initWithFields:objectFields withType:ListingTypeSponsored];
        [items addObject:object];
    }
    _sponsoredListings = items;
    return _sponsoredListings;
}


- (NSString*) sponsoredListingsAttribution
{
    NSDictionary *listings = [self.results dictionaryForKey:@"paid_listings"];
    
    if (listings == nil)
        return @"";
    
    id complexObject = [listings objectForKey:@"attribution"];
 
    if (complexObject == nil)
        return @"";
   
    NSArray *attributions = [xAdUtils objectsFromObject:complexObject];
    
    if (attributions == nil || attributions.count == 0)
        return @"";
    
    NSDictionary *attribution = [attributions objectAtIndex:0];
    
    return [attribution stringForKey:@"text"];
}


- (BOOL) hasSponsoredListings
{
    return [self.results boolForKey:@"p_listings"];
}


- (BOOL) hasOrganicListings
{
    return [self.results boolForKey:@"o_listings"];
}


- (NSString*) detailDescription
{
    return [self.results stringForKey:@"edesc"];
}


- (NSUInteger) organicsStartIndex
{
    NSDictionary *listings = [self.results dictionaryForKey:@"organic_listings"];

    if (listings == nil)
        return 0;
    
    return [listings unsignedIntegerForKey:@"s_index"];
}


- (NSUInteger) organicsTotalCount
{
    NSDictionary *listings = [self.results dictionaryForKey:@"organic_listings"];
    
    if (listings == nil)
        return 0;
    
    return [listings unsignedIntegerForKey:@"av_count"];
}


- (NSUInteger) organicsCount
{
    NSDictionary *listings = [self.results dictionaryForKey:@"organic_listings"];
    
    if (listings == nil)
        return 0;
    
    return [listings unsignedIntegerForKey:@"count"];
}


- (BOOL) isNextAvailable
{
    return (self.organicsStartIndex + self.organicsCount < self.organicsTotalCount);
}


- (BOOL) isPrevAvailable
{
    return (self.organicsStartIndex > 0);
}




- (void) sendImpressionNotifications {
    
    NSMutableArray *listingIds = [[NSMutableArray alloc] init];
    
    for (xAdListing *listing in self.sponsoredListings) {
        [listingIds addObject:listing.listingID];
    }
    
    for (xAdListing *listing in self.organicListings) {
        [listingIds addObject:listing.listingID];
    }

    xAdNotificationRequest *request = [[xAdNotificationRequest alloc] initImpressionWithListingIds: listingIds];
    
    [request execute];
}



@end