//
//  SocialLocation.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import "xAdLocation.h"
#import "NSDictionaryExtension.h"
#import "NSStringExtension.h"
#import "xAdSdk.h"




@implementation xAdLocation
@synthesize geoLocation = _geoLocation;
@synthesize addressInTwoLines = _addressInTwoLines;
@synthesize address = _address;


- (id) city
{
    return [self.fields stringForKey:@"city"];
}


- (id) country
{
    return [self.fields stringForKey:@"country"];
}


- (id) state
{
    return [self.fields stringForKey:@"state"];
}


- (id) street
{
    return [self.fields stringForKey:@"street"];
}


- (id) zip
{
    return [self.fields stringForKey:@"zip"];
}


- (BOOL) hasAddress
{
    BOOL hasStreet = (self.street != nil && [self.street length] > 0);
    BOOL hasCity = (self.city != nil && [self.city length] > 0);;
    BOOL hasState = (self.state != nil && [self.state length] > 0);;
    BOOL hasZip = (self.zip != nil && [self.zip length] > 0);;
    BOOL hasCountry = (self.country != nil && [self.country length] > 0);;
    
    return (hasStreet || hasCity || hasState || hasZip || hasCountry);
}


- (NSString*) addressInTwoLines
{
    if (_addressInTwoLines)
        return _addressInTwoLines;
    
    BOOL hasStreet = (self.street != nil && [self.street length] > 0);
    BOOL hasCity = (self.city != nil && [self.city length] > 0); 
    BOOL hasState = (self.state != nil && [self.state length] > 0); 
    
    id place = nil;
    
    if (hasCity && hasState)
        place = [NSString stringWithFormat:@"%@, %@", [self city], [self state]];
    else if (!hasCity && hasState)
        place = [NSString stringWithFormat:@"%@", [self state]];
    else if (hasCity && !hasState)
        place = [NSString stringWithFormat:@"%@", [self city]];
    
    BOOL hasPlace = (place != nil && [place length] > 0);
    
    [NSString stringWithFormat:@"%@, %@ %@", [self city], [self state], [self zip]];
    
    if (hasStreet && hasPlace)
        _addressInTwoLines = [NSString stringWithFormat:@"%@\n%@", self.street, place];
    else if (hasStreet && !hasPlace)
        _addressInTwoLines = [NSString stringWithFormat:@"%@", self.street];
    else if (!hasStreet && hasPlace)
         _addressInTwoLines = [NSString stringWithFormat:@"%@", place];
    else
        _addressInTwoLines = @"No address information";
    
    return _addressInTwoLines;
}


- (NSString*) address
{
    if (_address)
        return _address;
    
    BOOL hasStreet = (self.street != nil && [self.street length] > 0);
    BOOL hasCity = (self.city != nil && [self.city length] > 0);;
    BOOL hasState = (self.state != nil && [self.state length] > 0);;
    BOOL hasZip = (self.zip != nil && [self.zip length] > 0);;
    BOOL hasCountry = (self.country != nil && [self.country length] > 0);;
    
    if (hasStreet && hasCity)
        _address = [NSString stringWithFormat:@"%@, %@", self.street, self.city];
    else if (hasStreet && !hasCity && hasZip)
        _address = [NSString stringWithFormat:@"%@, %@", self.street, self.zip];
    else if (!hasStreet && hasCity && hasState && hasZip)
        _address = [NSString stringWithFormat:@"%@, %@ %@", self.city, self.state, self.zip];
    else if (!hasStreet && hasCity && hasState && !hasZip)
        _address = [NSString stringWithFormat:@"%@, %@", self.city, self.state];
    else if (!hasStreet && !hasCity && hasZip && hasState)
        _address = [NSString stringWithFormat:@"%@, %@", self.zip, self.state];
    else if (!hasStreet && !hasCity && !hasZip && hasState && hasCountry)
        _address = [NSString stringWithFormat:@"%@, %@", self.state, self.country];
    else if (!hasStreet && !hasCity && !hasZip  && !hasState && hasCountry)
        _address = [NSString stringWithFormat:@"%@", self.country];
    else         
        _address = @"No address information";
    
    return _address;
}


- (BOOL) hasCoordinates
{
    return (self.geoLocation.coordinate.latitude != 0 && self.geoLocation.coordinate.longitude != 0);
}


- (CLLocationDistance) currentDistance 
{ 
    id currentLocation = [xAdSdk currentLocation];
    if (self.geoLocation && currentLocation)
        return ([self.geoLocation distanceFromLocation:currentLocation] * 0.000621371192);
    else
        return -1;
}


- (CLLocationDistance) distance 
{ 
    id distance = [self.fields objectForKey:@"distance"];

    if (distance == nil)
    {
        id currentLocation = [xAdSdk currentLocation];
        if (self.geoLocation && currentLocation)
            return ([self.geoLocation distanceFromLocation:currentLocation] * 0.000621371192);
        else
            return -1;
    }
    else  
        return [self.fields doubleForKey:@"distance"]; 
}


- (id) distanceFormatted
{
    if (self.distance > 0.0f)
        return [NSString stringWithFormat:@"%1.2f miles", self.distance];
    else
        return nil;
}



- (CLLocation*) geoLocation
{
    if (_geoLocation)
        return _geoLocation;
    
    if ([self.fields objectForKey:@"latitude"] == nil || [self.fields objectForKey:@"longitude"] == nil)
        return nil;
    
    CLLocationDegrees lat = [self.fields doubleForKey:@"latitude"];
    CLLocationDegrees lon = [self.fields doubleForKey:@"longitude"];
    
    _geoLocation = [[CLLocation alloc] initWithLatitude:lat longitude:lon];
    
    
    return _geoLocation;
}



- (NSURL*)mapsUrl
{
    id value = [NSString stringWithFormat:@"http://maps.google.com/maps?ll=%f,%f", 
                self.geoLocation.coordinate.latitude, 
                self.geoLocation.coordinate.longitude];
    
    return [NSURL URLWithString:value];
}


- (NSURL*)directionsUrl
{
    NSString *destination = nil;
    if ([self hasCoordinates])
    {
        destination = [NSString stringWithFormat:@"%f,%f", self.geoLocation.coordinate.latitude, self.geoLocation.coordinate.longitude];
    }
    else if ([self hasAddress])
    {
        destination = self.address;
    }
    
    if (destination == nil)
        return nil;
    
    id value = [NSString stringWithFormat:@"http://maps.google.com/maps?saddr=Current+Location&daddr=%@", [destination urlEncode]];

    return [NSURL URLWithString:value];
}





@end
