//
//  xAdResponse.h
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import "JsonObject.h"

typedef enum
{
    xAdResponseFailure = -1,
    xAdResponseSuccess = 0,
    xAdResponseNetworkError = 101,
    xAdResponseRestError = 102,
    xAdResponseResourceNotAvailable = 103,
    xAdResponseMissingInput = 104,
    xAdResponseMissingKey = 105,
    xAdResponseMissingUid = 106,
    xAdResponseMissingVersion = 107,
    xAdResponseMissingNotifyType = 108,
    xAdResponseMissingListingId = 109,
    xAdResponseInvalidNumAds = 110,
    xAdResponseMissingLastName = 111,
    xAdResponseInvalidNumResults = 112,
    xAdResponseInvalidStartIndex = 113,
    xAdResponseInvalidUniqueIdLength = 114,
    xAdResponseInvalidCountryNameLength = 115,
    xAdResponseInvalidInputLength = 116,
    xAdResponseInvalidLatitude = 117,
    xAdResponseInvalidLongitude = 118,
    xAdResponseAppIdLengthExceeded = 119,
    xAdResponseFirstNameLengthExceeded = 120,
    xAdResponseLastNameLengthExceeded = 121,
    xAdResponseIPLengthExceeded = 122,
    xAdResponseUNKNOWN_REASON_1 = 123,
    xAdResponseNotifyTypeNotSupported = 124,
    xAdResponseServerError = 301,
    xAdResponseNoResults = 302,
    xAdResponseInvalidLocation = 303,
    xAdResponseListingExpired = 304,
    xAdResponseInvalidListingId = 305,
    xAdResponseInvalidCountry = 306,
    xAdResponseLDAPError = 307,
    xAdResponseUnableToRegister = 308,
    xAdResponseInvalidSuggestiveVersion = 309,
    xAdResponseSuggestiveVersionInSync = 310,
    xAdResponseMissingVoiceFormat = 1001,
    xAdResponseMissingVoiceData = 1002,
    xAdResponseMissingVoiceType = 1003,
    xAdResponseInvalidClientWidth = 1004,
    xAdResponseInvalidClientHeight = 1005,
    xAdResponseInvalidClientVersion = 1006,
    xAdResponseInvalidRegValue = 1007,
    xAdResponseInvalidDirectionType = 1008,
    xAdResponseInvalidVoiceDataFormat = 1009,
    xAdResponseInvalidName = 1010,
    xAdResponseInvalidEmail = 1011,
    xAdResponseInvalidMessage = 1012,
    xAdResponseInvalidSearchOptions = 1013
} xAdResponseCode;


//
// The base class of all xAd Responses, including error responses.
//
@interface xAdResponse : JsonObject

// All JSON responses from xAd have a 'results' key. This is it!
@property (nonatomic, readonly) NSDictionary *results;


// The request ID sent in the request
@property (nonatomic, readonly) NSString *requestId;


// Determines from the responseCode if the response was good.
@property (nonatomic, readonly) BOOL isSuccess;


// Response codes - see enum above.
@property (nonatomic, readonly) xAdResponseCode responseCode;


// In most cases there will be a description sent from the server.
@property (nonatomic, readonly) NSString *errorDescription;


// Even the error responses have the UID sent in the invoking request
@property (nonatomic, readonly) NSString *uid;


// Constructs a NSError using the information from the response.
// If the response was not an error, this method returns nil.
- (NSError*) createError;



@end
