//
//  NSDateExtension.h
//
//  Licensed by ruralcoder.com under the
//  Creative Commons Attribution-ShareAlike 3.0 Unported License
//

#import "NSDateExtension.h"

@implementation NSDate (Extension)


+ (NSDate*) dateFromUtcString:(NSString*)formattedDate 
{
	return [NSDate dateFromString:formattedDate format:@"yyyy-MM-dd HH:mm:ss" timezone:@"UTC"];
}


+ (NSDate*) dateFromString:(NSString*)formattedDate format:(NSString*)format timezone:(id)timezoneName
{
	NSDateFormatter* dateFormatter = [[NSDateFormatter alloc] init];
	
    id timezone = [NSTimeZone timeZoneWithName:timezoneName];
    
    if (timezone)
        [dateFormatter setTimeZone:timezone];
    
    [dateFormatter setFormatterBehavior:NSDateFormatterBehaviorDefault];
	[dateFormatter setDateFormat:format];
    [dateFormatter setLocale:[NSLocale systemLocale]];

	return [dateFormatter dateFromString:formattedDate];
}


+ (NSString*) stringFromTime:(NSDate*)date
{
	NSLocale *curLocale = [NSLocale currentLocale];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	
	[dateFormatter setTimeStyle:NSDateFormatterShortStyle];
	[dateFormatter setDateStyle:NSDateFormatterNoStyle ];
	[dateFormatter setLocale:curLocale];

	return [dateFormatter stringFromDate:date];
}


+ (NSString*) stringFromDate:(NSDate*)date
{
	NSLocale *curLocale = [NSLocale currentLocale];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	
	[dateFormatter setTimeStyle:NSDateFormatterNoStyle];
	[dateFormatter setDateStyle:NSDateFormatterShortStyle ];
	[dateFormatter setLocale:curLocale];

	return [dateFormatter stringFromDate:date];
}



+ (NSString*) stringFromTimestamp:(NSDate*)date
{
	NSLocale *curLocale = [NSLocale currentLocale];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	
	[dateFormatter setTimeStyle:NSDateFormatterShortStyle ];
	[dateFormatter setDateStyle:NSDateFormatterMediumStyle ];
	[dateFormatter setLocale:curLocale];

	return [dateFormatter stringFromDate:date];
}


- (NSString*) toDateString:(NSDateFormatterStyle)style
{
	NSLocale *curLocale = [NSLocale currentLocale];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	
	[dateFormatter setDateStyle:style ];
	[dateFormatter setLocale:curLocale];

	return [dateFormatter stringFromDate:self];
}

 
- (NSString*) toTimeString:(NSDateFormatterStyle)style
{
	NSLocale *curLocale = [NSLocale currentLocale];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	
	[dateFormatter setTimeStyle:style ];
	[dateFormatter setLocale:curLocale];

	return [dateFormatter stringFromDate:self];
}

- (NSString*) toTimestampString:(NSDateFormatterStyle)style;
{
	NSLocale *curLocale = [NSLocale currentLocale];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	
	[dateFormatter setTimeStyle:NSDateFormatterShortStyle];
	[dateFormatter setDateStyle:style ];
	[dateFormatter setLocale:curLocale];
	
	return [dateFormatter stringFromDate:self];
}


@end
