//
//  xAdUtils.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import "xAdUtils.h"
#import "xAdSdk.h"

#import <UIKit/UIWebView.h>
#import <CoreTelephony/CTTelephonyNetworkInfo.h>
#import <CoreTelephony/CTCarrier.h>
#import <AdSupport/ASIdentifierManager.h>
#import <SystemConfiguration/CaptiveNetwork.h>

#import <sys/utsname.h>
#import <ifaddrs.h>
#import <arpa/inet.h>



@implementation xAdUtils


+ (id) applicationName {
    return [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleDisplayName"];
}


+ (id) applicationVersion {
    return [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleVersion"];
}


+ (id) userAgent {
    UIWebView* webView = [[UIWebView alloc] initWithFrame:CGRectZero];
    return [webView stringByEvaluatingJavaScriptFromString:@"navigator.userAgent"];
}


+ (id) deviceWifi {
#ifdef _SYSTEMCONFIGURATION_H

    CFArrayRef interfaces = CNCopySupportedInterfaces();
    
    if (!interfaces)
        return @"";
    
    NSDictionary* info = (__bridge NSDictionary *) CNCopyCurrentNetworkInfo(CFArrayGetValueAtIndex(interfaces, 0));
    
    if (!info)
        return @"";
    
    id value = [info valueForKey:@"SSID"];
    
    if (!value)
        return @"";
    
    return value;
    
#else
    return @"";
#endif
}


+ (id) deviceOS {
    NSDictionary *systemVersion = [NSDictionary dictionaryWithContentsOfFile:@"/System/Library/CoreServices/SystemVersion.plist"];
    
    return [systemVersion objectForKey:@"ProductVersion"];
}


+ (id) deviceLanguage {
    return [[NSLocale preferredLanguages] objectAtIndex:0];
}


+ (id) deviceType {
    
    switch ([[UIDevice currentDevice] userInterfaceIdiom]) {
        case UIUserInterfaceIdiomPad:
            return @"tablet";
            
        case UIUserInterfaceIdiomPhone:
        default:
            return @"phone";
    }
}


+ (id) deviceModel {
    struct utsname systemInfo;
    uname(&systemInfo);
    
    return [NSString stringWithCString:systemInfo.machine encoding:NSUTF8StringEncoding];
}


+ (id) carrier {

    CTTelephonyNetworkInfo *netInfo = [[CTTelephonyNetworkInfo alloc] init];
    CTCarrier *carrier = [netInfo subscriberCellularProvider];
    
    NSString *mcc = [carrier mobileCountryCode];
    NSString *mnc = [carrier mobileNetworkCode];
    NSString *carrierName = [carrier carrierName];
   
    id value = nil;
    if (mcc && mnc) {
        value = [NSString stringWithFormat:@"%@-%@", mcc, mnc];
    } else if (carrierName) {
        value = carrierName;
    } else {
        value = @"simulator";
    }

    NSLog(@"%@", value);
    
    return value;
}


+ (id) advertisingIdentifier {
    return [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString];
}


+ (id) isDoNotTrackEnabled {
    return [[ASIdentifierManager sharedManager] isAdvertisingTrackingEnabled] ? @"0" : @"1";
}


+ (NSString*) toURL:(NSDictionary*)args urlEncode:(BOOL)encode
{
    NSMutableArray *urlParams = [NSMutableArray array];
    
    
    NSArray *keys = [args allKeys];
    for (id key in keys)
    {
        id value = [args objectForKey:key];
        
        // Lets check if it is an array - used for listing ids.
        if ([value isKindOfClass:[NSArray class]]) {
            
            for (id singleValue in value) {
                // Same key, but with repeats
                id pair = [NSString stringWithFormat:@"%@=%@", key, singleValue];
                
                [urlParams addObject:pair];
            }
            
        } else {
            // Lets encode only string parameters
            if ([value isKindOfClass:[NSString class]] && encode) {
                value = [value stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
            } else if ([value isKindOfClass:[NSNumber class]]) {
                value = [value stringValue];
            }

            id pair = [NSString stringWithFormat:@"%@=%@", key, value];
            
            [urlParams addObject:pair];
        }
    }
    
    return [urlParams componentsJoinedByString:@"&"];
}


+ (id) deviceIp
{
    struct ifaddrs *interfaces = NULL;
    struct ifaddrs *currentAddress = NULL;
    NSString *wifiAddress = nil;
    NSString *cellAddress = nil;
    
    BOOL onFreeNetwork = FALSE;
    
    // retrieve the current interfaces - returns 0 on success
    if(getifaddrs(&interfaces)) {
        NSLog(@"Failed to retrieve network interfaces");
        return nil;
    }
    
    // Loop through linked list of interfaces
    currentAddress = interfaces;
    while(currentAddress != NULL)
    {
        sa_family_t addressFamily = currentAddress->ifa_addr->sa_family;
        
        // Check address family
        if(addressFamily != AF_INET && addressFamily != AF_INET6) {
            NSLog(@"Invalid address family %d", addressFamily);
            currentAddress = currentAddress->ifa_next;
            continue;
        }

        NSString *name = [NSString stringWithUTF8String:currentAddress->ifa_name];
        NSString *addr = [NSString stringWithUTF8String:inet_ntoa(((struct sockaddr_in *)currentAddress->ifa_addr)->sin_addr)]; // pdp_ip0
        
        // Check localhost of disabled interfaces
        if ([addr isEqualToString:@"0.0.0.0"] || [addr isEqualToString:@"127.0.0.1"]) {
            currentAddress = currentAddress->ifa_next;
            continue;
        }
        
        // Nice, we found something
        NSLog(@"'%@' => %@", name, addr);
        
        if ([name hasPrefix:@"en"]) {
            wifiAddress = addr;
            onFreeNetwork = TRUE;
        }
        
        if([name isEqualToString:@"en0"]) { // Interface is the wifi connection on the iPhone
            wifiAddress = addr;
            onFreeNetwork = TRUE;
        }
        else if([name isEqualToString:@"pdp_ip0"]) { // Interface is the cell connection on the iPhone
            cellAddress = addr;
        }

        currentAddress = currentAddress->ifa_next;
    }

    // Free memory
    freeifaddrs(interfaces);

    if (onFreeNetwork) {
        NSLog(@"onFreeNetwork");
    }
    
    return wifiAddress ? wifiAddress : cellAddress;
}


+ (void) downloadResource:(NSURL*)url notificationName:(NSString*)notificationName
{
    NSLog(@"downloadResource: %@", url.absoluteString);
    
    dispatch_async(KBackgroundQueue, ^(void)
    {
        NSError *networkError = nil;
        NSData* data = [NSData dataWithContentsOfURL:url options:NSDataReadingUncached error:&networkError];
                       
        dispatch_async(KForegroundQueue, ^(void) {
        
            if (networkError) {
                NSLog(@"networkError: %@", [networkError localizedDescription]);
                [[NSNotificationCenter defaultCenter] postNotificationName:notificationName object:networkError];
                return;
            }

            [[NSNotificationCenter defaultCenter] postNotificationName:notificationName object:data];
        });
                       
    });
}


+ (NSArray*) objectsFromObject:(id)complexObject
{
    NSMutableArray *objects = [NSMutableArray array];
    
    if (complexObject == nil)
        return objects;
    
    if ([complexObject isKindOfClass:[NSArray class]])
        [objects addObjectsFromArray:complexObject];
    else
        [objects addObject:complexObject];
    
    return objects;
}


+ (void) dialPhonenumber:(id)phoneNumber {
    
    if (!phoneNumber) {
        return;
    }
    
    NSURL *telephonyUrl = [NSURL URLWithString:[NSString stringWithFormat:@"tel://%@", phoneNumber]];
    
    [[UIApplication sharedApplication] openURL:telephonyUrl];
}


@end

