//
//  xAdSdk.h
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreLocation/CoreLocation.h>

#define KForegroundQueue dispatch_get_main_queue()
#define KBackgroundQueue dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0)


// Search Only
UIKIT_EXTERN NSString *const xAdParameterSearchListings;
UIKIT_EXTERN NSString *const xAdParameterSearchRequestId;
UIKIT_EXTERN NSString *const xAdParameterSearchNumberOfAds;
UIKIT_EXTERN NSString *const xAdParameterSearchAdPage;
UIKIT_EXTERN NSString *const xAdParameterSearchStreetNumber;
UIKIT_EXTERN NSString *const xAdParameterSearchStreetName;
UIKIT_EXTERN NSString *const xAdParameterSearchStreetCity;


// Search Organics
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsCategories;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsStartIndex;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsNumberOfItems;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsSort;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsFilterRadius;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsFilterCategory;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsFilterCategoryType;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsFilterRating;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsFilterAlpha;
UIKIT_EXTERN NSString *const xAdParameterSearchOrganicsFilterPrice;


// Common
UIKIT_EXTERN NSString *const xAdParameterVersion;
UIKIT_EXTERN NSString *const xAdParameterPublisherKey;
UIKIT_EXTERN NSString *const xAdParameterPublisherTrafficSource;
UIKIT_EXTERN NSString *const xAdParameterUserId;
UIKIT_EXTERN NSString *const xAdParameterDeviceUserAgent;
UIKIT_EXTERN NSString *const xAdParameterAdInput;
UIKIT_EXTERN NSString *const xAdParameterGeoLat;
UIKIT_EXTERN NSString *const xAdParameterGeoLong;
UIKIT_EXTERN NSString *const xAdParameterGeoCityState;
UIKIT_EXTERN NSString *const xAdParameterGeoCountry;
UIKIT_EXTERN NSString *const xAdParameterAdFormat;
UIKIT_EXTERN NSString *const xAdParameterDeviceIp;


// Display Only
UIKIT_EXTERN NSString *const xAdParameterDisplayApi;
UIKIT_EXTERN NSString *const xAdParameterDisplaySize;


// Extensions
UIKIT_EXTERN NSString *const xAdParameterPublisherType;
UIKIT_EXTERN NSString *const xAdParameterDeviceType;
UIKIT_EXTERN NSString *const xAdParameterDeviceModel;
UIKIT_EXTERN NSString *const xAdParameterDeviceLanguage;
UIKIT_EXTERN NSString *const xAdParameterDeviceCarrier;
UIKIT_EXTERN NSString *const xAdParameterDeviceWifiProvider;
UIKIT_EXTERN NSString *const xAdParameterDeviceOS;
UIKIT_EXTERN NSString *const xAdParameterGeoZip;
UIKIT_EXTERN NSString *const xAdParameterGeoPolygon;
UIKIT_EXTERN NSString *const xAdParameterUserIdType;
UIKIT_EXTERN NSString *const xAdParameterUserDoNotTrack;
UIKIT_EXTERN NSString *const xAdParameterUserAge;
UIKIT_EXTERN NSString *const xAdParameterUserGender;
UIKIT_EXTERN NSString *const xAdParameterAdCategory;

// Notifications
UIKIT_EXTERN NSString *const xAdParameterSearchListingId;
UIKIT_EXTERN NSString *const xAdParameterSearchListingInfo;
UIKIT_EXTERN NSString *const xAdParameterSearchRequestType;


// Notification Center Events
UIKIT_EXTERN NSString *const xAdEventSDKError;
UIKIT_EXTERN NSString *const xAdEventLocationChanged;
UIKIT_EXTERN NSString *const xAdEventDisplayAdRequestFailed;
UIKIT_EXTERN NSString *const xAdEventDisplayAdClicked;
UIKIT_EXTERN NSString *const xAdEventDisplayAdSecondary;
UIKIT_EXTERN NSString *const xAdEventDisplayAdNoFill;
UIKIT_EXTERN NSString *const xAdEventDisplayAdFilled;

UIKIT_EXTERN NSString *const xAdEventSearchCompleted;
UIKIT_EXTERN NSString *const xAdEventSearchNotifyCompleted;


@class CLLocationManager;

@interface xAdSdk : NSObject <CLLocationManagerDelegate>

+ (xAdSdk *) sharedInstance;

+ (CLLocationManager*) locationManager;
+ (CLLocation *) currentLocation;
    

// Location Targeting enables location services and sends to xAd the lat/long of the user.
// If not enabled, publisher must provide alternative location information
- (void) disableCoreLocation;
- (void) enableProduction;



// Returns the search production or staging server depending on whether enableProduction was called
@property (readonly, nonatomic) NSString* searchServer;


// Returns the display production or staging server depending on whether enableProduction was called
@property (readonly, nonatomic) NSString* displayServer;


// Allows you to assign your publisher key for display banners provided by xAd.
@property (strong, nonatomic) NSString* displayPublisherKey;


// Allows you to assign your publisher key for search ads provided by xAd.
@property (strong, nonatomic) NSString* searchPublisherKey;


// Starts background services.
- (void) startServices;


// Suspend background services.
- (void) suspendServices;


@end



