#import "xAdSdk.h"

#import <CoreLocation/CoreLocation.h>
#import <CoreMotion/CoreMotion.h>

#import "TargetConditionals.h"

// Search Only
NSString *const xAdParameterSearchListings = @"l";
NSString *const xAdParameterSearchRequestId = @"reqid";
NSString *const xAdParameterSearchNumberOfAds = @"n_ad";
NSString *const xAdParameterSearchAdPage = @"ref";
NSString *const xAdParameterSearchStreetNumber = @"s_num";
NSString *const xAdParameterSearchStreetName = @"s_name";
NSString *const xAdParameterSearchStreetCity = @"s_city";


// Search Organics
NSString *const xAdParameterSearchOrganicsCategories = @"f_cat";
NSString *const xAdParameterSearchOrganicsStartIndex = @"s";
NSString *const xAdParameterSearchOrganicsNumberOfItems = @"n";
NSString *const xAdParameterSearchOrganicsSort = @"sort";
NSString *const xAdParameterSearchOrganicsFilterRadius = @"f_radius";
NSString *const xAdParameterSearchOrganicsFilterCategory = @"f_cat";
NSString *const xAdParameterSearchOrganicsFilterCategoryType = @"f_cat_type";
NSString *const xAdParameterSearchOrganicsFilterRating = @"f_rating";
NSString *const xAdParameterSearchOrganicsFilterAlpha = @"f_aplha";
NSString *const xAdParameterSearchOrganicsFilterPrice = @"f_price";


// Common
NSString *const xAdParameterVersion = @"v";
NSString *const xAdParameterPublisherKey = @"k";
NSString *const xAdParameterPublisherTrafficSource = @"appid";
NSString *const xAdParameterUserId = @"uid";
NSString *const xAdParameterDeviceUserAgent = @"devid";
NSString *const xAdParameterAdInput = @"i";
NSString *const xAdParameterGeoLat = @"lat";
NSString *const xAdParameterGeoLong = @"long";
NSString *const xAdParameterGeoCityState = @"loc";
NSString *const xAdParameterGeoCountry = @"co";
NSString *const xAdParameterAdFormat = @"o_fmt";
NSString *const xAdParameterDeviceIp = @"ip";


// Display Only
NSString *const xAdParameterDisplayApi = @"api";
NSString *const xAdParameterDisplaySize = @"size";


// Extensions
NSString *const xAdParameterPublisherType = @"pt";
NSString *const xAdParameterDeviceType = @"dev_type";
NSString *const xAdParameterDeviceModel = @"dev_model";
NSString *const xAdParameterDeviceLanguage = @"lang";
NSString *const xAdParameterDeviceCarrier = @"carrier";
NSString *const xAdParameterDeviceWifiProvider = @"wifi";
NSString *const xAdParameterDeviceOS = @"os";
NSString *const xAdParameterGeoZip = @"zip";
NSString *const xAdParameterGeoPolygon = @"tl";
NSString *const xAdParameterUserIdType = @"uid_type";
NSString *const xAdParameterUserDoNotTrack = @"dnt";
NSString *const xAdParameterUserAge = @"age";
NSString *const xAdParameterUserGender = @"gender";
NSString *const xAdParameterAdCategory = @"category";

// Notifications
NSString *const xAdParameterSearchListingId = @"l_id";
NSString *const xAdParameterSearchListingInfo = @"l_info";
NSString *const xAdParameterSearchRequestType = @"t";


// Notification Center Events
NSString *const xAdEventLocationChanged = @"XAD_LOC_CHANGED";
NSString *const xAdEventDisplayAdRequestFailed = @"XAD_AD_FAILED";
NSString *const xAdEventDisplayAdClicked = @"XAD_AD_CLICKED";
NSString *const xAdEventDisplayAdSecondary = @"XAD_AD_SECOND";
NSString *const xAdEventDisplayAdNoFill = @"XAD_DISPLAY_NO_FILL";

NSString *const xAdEventSDKError = @"XAD_SDK_ERROR";
NSString *const xAdEventDisplayAdFilled = @"XAD_AD_FILLED";
NSString *const xAdEventSearchCompleted = @"XAD_SEARCH_COMPLETED";
NSString *const xAdEventSearchNotifyCompleted = @"XAD_NOTIFY_COMPLETED";



@interface xAdSdk ()
    @property (assign, nonatomic) BOOL featureUseCoreLocation;
    @property (assign, nonatomic) BOOL featureUseProductionServers;
    @property (strong, nonatomic) CLLocationManager *locationManager;
    @property (strong, nonatomic) CLLocation *cachedLocation;
@end



@implementation xAdSdk

@synthesize displayPublisherKey;
@synthesize searchPublisherKey;
@synthesize cachedLocation;

+ (xAdSdk *) sharedInstance {
    static xAdSdk *singleton = nil;
    
    @synchronized(self)
    {
        if (!singleton) {
            
            NSLog(@"xAdSdk created");
            
            singleton = [[xAdSdk alloc] init];

            // Default values
            singleton.featureUseCoreLocation = TRUE;
            singleton.featureUseProductionServers = FALSE;
            singleton.featureUseProductionServers = FALSE;
            
            // Defaults to ?
            singleton.cachedLocation = [[CLLocation alloc] initWithLatitude:40.780184 longitude:-73.966827];
        }
        
        return singleton;
    }
}


    
    
+ (CLLocationManager*) locationManager {
    return [[xAdSdk sharedInstance] locationManager];
}
   
    
+ (CLLocation*) currentLocation {
    return [[xAdSdk sharedInstance] cachedLocation];
}


    
- (void) startServices {

    NSLog(@"xAdSdk startServices");

    if (self.featureUseCoreLocation) {
        
        self.locationManager = [[CLLocationManager alloc] init];
        self.locationManager.delegate = self;
        
        self.locationManager.desiredAccuracy = kCLLocationAccuracyBestForNavigation;
        [self.locationManager startMonitoringSignificantLocationChanges];
        // If there is no core location service, trigger one event to proceed.
        if (![CLLocationManager locationServicesEnabled]) {
            [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventLocationChanged object:nil];
        }
    }
}


- (void) suspendServices {
    
    NSLog(@"xAdSdk suspendServices");

    if (self.featureUseCoreLocation) {
        [self.locationManager stopUpdatingLocation];
    }
}


- (void) disableCoreLocation {
    self.featureUseCoreLocation = FALSE;
}


- (void) enableProduction {
    self.featureUseProductionServers = TRUE;
}


- (NSString*) searchServer {
    
    if (self.featureUseProductionServers) {
        return @"http://local.xad.com/rest/%@?%@";
    } else {
        return @"http://test-search.xad.com/rest/%@?%@";
    }
}


- (NSString*) displayServer {
    
    if (self.featureUseProductionServers) {
        return @"http://display.xad.com/rest/banner?%@";
    } else {
        // return @"http:/test-display.xad.com/rest/banner?%@";
        return @"http:/test.xad.com:8085/rest/banner?%@";
    }
}


#pragma mark Core Location Events

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error
{
    NSLog(@"locationManager:didFailWithError%@", [error description]);
    
    [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventLocationChanged object:nil];
}


- (void)locationManager:(CLLocationManager *)manager
    didUpdateToLocation:(CLLocation *)newLocation
           fromLocation:(CLLocation *)oldLocation
{
    
    if (newLocation.coordinate.longitude == oldLocation.coordinate.longitude &&
        newLocation.coordinate.latitude == oldLocation.coordinate.latitude &&
        newLocation.horizontalAccuracy == oldLocation.horizontalAccuracy)
    {
        return;
    }
    
    // Check accuracy
    
    CLLocationDistance distance = ([newLocation distanceFromLocation:oldLocation]) * 0.000621371192;
    
    // Added due to minor fluctuations of the location perhaps due to hand movements.
    if (oldLocation && distance < .001) {
        return;
    }
    
    NSLog(@"locationManager:didUpdateToLocation");
    NSLog(@"Distance: %f miles",distance);
    
    NSLog(@"Location %@",newLocation);
    
    self.cachedLocation = newLocation;

    [[NSNotificationCenter defaultCenter] postNotificationName:xAdEventLocationChanged object:newLocation];
}



@end

